@@ -4,6 +4,7 @@ package Net::BitTorrent;
     use strict;
     use warnings;
     use Scalar::Util qw[blessed weaken refaddr];
+    use List::Util qw[max];
     use Time::HiRes;
     use Socket qw[/inet_/ SOCK_STREAM SOCK_DGRAM SOL_SOCKET PF_INET SOMAXCONN
         /pack_sockaddr_in/];
@@ -17,15 +18,18 @@ package Net::BitTorrent;
     use Net::BitTorrent::DHT;
     use Net::BitTorrent::Version;
     use version qw[qv];
-    our $SVN = q[$Id: BitTorrent.pm 33 2008-11-10 23:27:24Z sanko@cpan.org $];
-    our $UNSTABLE_RELEASE = 4; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 33 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
+    our $SVN = q[$Id: BitTorrent.pm 34 2008-11-20 03:38:52Z sanko@cpan.org $];
+    our $UNSTABLE_RELEASE = 4; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 34 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
     my (@CONTENTS)
-        = \my (%_tcp,         %_udp,               %_peerid,
-               %_schedule,    %_peers_per_torrent, %_event,
-               %_dht,         %_torrents,          %_connections,
-               %_max_ul_rate, %_k_up,              %_max_dl_rate,
-               %_k_down,      %_tid,               %_use_dht,
-               %__UDP_OBJECT_CACHE
+        = \my (%_tcp,               %_udp,
+               %_peerid,            %_schedule,
+               %_event,             %_torrents,
+               %_connections,       %_max_ul_rate,
+               %_k_ul,              %_max_dl_rate,
+               %_k_dl,              %_dht,
+               %_tid,               %_use_dht,
+               %__UDP_OBJECT_CACHE, %_peers_per_torrent,
+               %_connections_per_host
         );
     my %REGISTRY;
 
@@ -49,24 +53,20 @@ package Net::BitTorrent;
                 )
                 : @ports;
         }
-        $_peers_per_torrent{refaddr $self} = 50;
-        $_max_dl_rate{refaddr $self}       = 0;
-        $_max_ul_rate{refaddr $self}       = 0;
-        $_torrents{refaddr $self}          = {};
-        $_tid{refaddr $self}               = qq[\0] x 5;
-        $_use_dht{refaddr $self}           = 1;
+        $_peers_per_torrent{refaddr $self}    = 50;
+        $_connections_per_host{refaddr $self} = 3;
+        $_max_dl_rate{refaddr $self}          = 0;
+        $_k_dl{refaddr $self}                 = 0;
+        $_max_ul_rate{refaddr $self}          = 0;
+        $_k_ul{refaddr $self}                 = 0;
+        $_torrents{refaddr $self}             = {};
+        $_tid{refaddr $self}                  = qq[\0] x 5;
+        $_use_dht{refaddr $self}              = 1;
         $_dht{refaddr $self} = Net::BitTorrent::DHT->new({Client => $self});
         $_peerid{refaddr $self}      = Net::BitTorrent::Version::gen_peerid();
         $_connections{refaddr $self} = {};
+        $self->_reset_bandwidth;
         for my $port (@ports) { last if $self->_socket_open($host, $port) }
-        $self->_schedule(
-            {Time => time + 5,
-             Code => sub {
-                 return $_k_down{refaddr $_[0]} = $_k_up{refaddr $_[0]} = 0;
-             },
-             Object => $self
-            }
-        );
         weaken($REGISTRY{refaddr $self} = $self);
         return $self;
     }
@@ -111,10 +111,14 @@ package Net::BitTorrent;
             = unpack_sockaddr_in(getsockname($_udp{refaddr $self}));
         return inet_ntoa($packed_ip);
     }
-    sub _peers_per_torrent { return $_peers_per_torrent{refaddr +shift}; }
+    sub _peers_per_torrent { return $_peers_per_torrent{refaddr +shift} }
+
+    sub _connections_per_host {
+        return $_connections_per_host{refaddr +shift};
+    }
 
     # Setters | Private
-    sub _set_max_ul_rate {
+    sub _set_max_ul_rate {    # BYTES per second
         my ($self, $value) = @_;
         if (not defined $value or $value !~ m[^\d+$]) {
             carp
@@ -124,7 +128,7 @@ package Net::BitTorrent;
         return $_max_ul_rate{refaddr $self} = $value;
     }
 
-    sub _set_max_dl_rate {
+    sub _set_max_dl_rate {    # BYTES per second
         my ($self, $value) = @_;
         if (not defined $value or $value !~ m[^\d+$]) {
             carp
@@ -151,9 +155,12 @@ package Net::BitTorrent;
     # Methods | Public
     sub do_one_loop {
         my ($self, $timeout) = @_;
-        return if defined $timeout and $timeout !~ m[^\+?\d+\.?\d*$];
+        $timeout = -1 if scalar @_ != 2;
+        return if defined $timeout and $timeout !~ m[^\-?\d+\.?\d*$];
         $self->_process_schedule;
         $timeout = ($timeout ? $timeout : 1);
+        $timeout = undef if $timeout == -1;
+        return if defined $timeout and $timeout < 0;
         my ($rin, $win, $ein) = (q[], q[], q[]);
     PUSHSOCK: for my $fileno (keys %{$_connections{refaddr $self}}) {
             vec($rin, $fileno, 1) = 1
@@ -163,14 +170,26 @@ package Net::BitTorrent;
             vec($ein, $fileno, 1) = 1;
         }
         my ($nfound, $timeleft) = select($rin, $win, $ein, $timeout);
-        my $time = time + $timeleft;
         $self->_process_connections(\$rin, \$win, \$ein)
             if $nfound and $nfound != -1;
-        if (($time - time) > 0) { sleep($time - time) }
-        return 1;
+        return $timeleft;
     }
 
     # Methods | Private
+    sub _reset_bandwidth {
+        my ($self) = @_;
+        $self->_schedule({Time   => time + 1,
+                          Code   => \&_reset_bandwidth,
+                          Object => $self
+                         }
+        );
+
+        #warn sprintf q[Speed report: Up: %5dB/s | Down: %5dB/s],
+        #    $_k_ul{refaddr $_[0]},
+        #    $_k_dl{refaddr $_[0]};
+        return $_k_dl{refaddr $_[0]} = $_k_ul{refaddr $_[0]} = 0;
+    }
+
     sub _add_connection {
         my ($self, $connection, $mode) = @_;
         if (not defined $connection) {
@@ -189,17 +208,10 @@ package Net::BitTorrent;
                 q[Net::BitTorrent->_add_connection(SOCKET, MODE) requires a mode parameter];
             return;
         }
-        if (    (fileno $_sock)
-            and (defined $_connections{refaddr $self}{fileno $_sock})
-            and
-            ($_connections{refaddr $self}{fileno $_sock}{q[Mode]} eq $mode))
-        {   return;
-        }
-        $_connections{refaddr $self}{fileno $_sock} = {Object => $connection,
-                                                       Mode   => $mode
-            }
-            or return;
-        return 1;
+        return $_connections{refaddr $self}{fileno $_sock} = {
+                                                        Object => $connection,
+                                                        Mode   => $mode
+        };
     }
 
     sub _remove_connection {
@@ -227,18 +239,22 @@ package Net::BitTorrent;
                 q[Net::BitTorrent->_socket_open(HOST, PORT) requires a blessed object];
             return;
         }
-        if (!$host) {
+        if ((!$_tcp{refaddr $self}) && (!$host)) {
             carp q[Net::BitTorrent::__socket_open( ) ]
                 . q[requires a hostname];
             return;
         }
-        if ((!defined $port) || ($port !~ m[^\d+$])) {
-            carp q[Net::BitTorrent::__socket_open( ) ]
+        if (   (!$_tcp{refaddr $self}) && (!defined $port)
+            || (defined $port and $port !~ m[^\d+$]))
+        {   carp q[Net::BitTorrent::__socket_open( ) ]
                 . q[requires an integer port number];
             return;
         }
         my $_packed_host = undef;
-        if ($host
+        $host ||= q[0.0.0.0];
+        $port ||= 0;
+        if (    $host
+            and $host
             !~ m[^(?:(?:(?:25[0-5]|2[0-4][0-9]|[0-1]?[0-9]{1,2})[.]?){4})$])
         {   my ($name, $aliases, $addrtype, $length, @addrs)
                 = gethostbyname($host)
@@ -246,14 +262,12 @@ package Net::BitTorrent;
             $_packed_host = $addrs[0];
         }
         else { $_packed_host = inet_aton($host) }
-
-        #
-        if (!$_tcp{refaddr $self}) {    # TCP
+        if (!$_tcp{refaddr $self}) {
             socket(my ($_tcp), PF_INET, SOCK_STREAM, getprotobyname(q[tcp]))
                 or return;
             bind($_tcp, pack_sockaddr_in($port, $_packed_host))
                 or return;
-            listen($_tcp, SOMAXCONN) or return;
+            listen($_tcp, 1) or return;
             $_connections{refaddr $self}{fileno $_tcp} = {Object => $self,
                                                           Mode   => q[ro]
                 }
@@ -276,11 +290,7 @@ package Net::BitTorrent;
         }
         if (    defined $_tcp{refaddr $self}
             and defined $_udp{refaddr $self})
-        {
-
-          # Even if the TCP and UDP sockets fail, we can still function, we're
-          # just not connectable.  Which isn't good but isn't fatal.
-            my ($tcp_port, $tcp_packed_ip)
+        {   my ($tcp_port, $tcp_packed_ip)
                 = unpack_sockaddr_in(getsockname($_tcp{refaddr $self}));
             my $tcp_address
                 = sprintf(q[%s:%d], inet_ntoa($tcp_packed_ip), $tcp_port);
@@ -290,8 +300,6 @@ package Net::BitTorrent;
                 = sprintf(q[%s:%d], inet_ntoa($udp_packed_ip), $udp_port);
             $$self = $tcp_address . q[ | ] . $udp_address;
         }
-
-        #
         return ($_tcp{refaddr $self} && $_udp{refaddr $self});
     }
 
@@ -303,31 +311,40 @@ package Net::BitTorrent;
             )
             )
         {   carp
-                q[Malformed parameters passed to Net::BitTorrent::_process_connections(RIN, WIN, EIN)];
+                q[Malformed parameters to Net::BitTorrent::_process_connections(RIN, WIN, EIN)];
             return;
         }
     POPSOCK: foreach my $fileno (keys %{$_connections{refaddr $self}}) {
             next POPSOCK unless defined $_connections{refaddr $self}{$fileno};
-            if ($fileno == fileno $_tcp{refaddr $self}) {
-                if (vec($$rin, $fileno, 1) == 1) {
+            if (   $_tcp{refaddr $self}
+                && $fileno == fileno $_tcp{refaddr $self})
+            {   if (vec($$rin, $fileno, 1) == 1) {
                     vec($$rin, $fileno, 1) = 0;
-                    accept(my ($new_socket), $_tcp{refaddr $self})
-                        or next POPSOCK;
-                    my $new_peer =
+                    if (scalar(
+                            grep {
+                                $_->{q[Object]}->isa(q[Net::BitTorrent::Peer])
+                                    && !$_->{q[Object]}->_torrent
+                                } values %{$_connections{refaddr $self}}
+                        ) < 8
+                        )
+                    {   accept(my ($new_socket), $_tcp{refaddr $self})
+                            or next POPSOCK;
                         Net::BitTorrent::Peer->new({Socket => $new_socket,
                                                     Client => $self
                                                    }
                         );
+                    }
                 }
             }
-            elsif ($fileno == fileno $_udp{refaddr $self}) {
-                if (vec($$rin, $fileno, 1) == 1) {
+            elsif (   $_udp{refaddr $self}
+                   && $fileno == fileno $_udp{refaddr $self})
+            {   if (vec($$rin, $fileno, 1) == 1) {
                     vec($$rin, $fileno, 1) = 0;
                     my $paddr
                         = recv($_udp{refaddr $self}, my ($data), 1024, 0)
                         or next POPSOCK;
-                    if ($__UDP_OBJECT_CACHE{refaddr $self}{$paddr}) {
-                        $__UDP_OBJECT_CACHE{refaddr $self}{$paddr}{q[Object]}
+                    if ($__UDP_OBJECT_CACHE{refaddr $self}{$paddr}{q[Object]})
+                    {   $__UDP_OBJECT_CACHE{refaddr $self}{$paddr}{q[Object]}
                             ->_on_data($paddr, $data)
                             or
                             delete $__UDP_OBJECT_CACHE{refaddr $self}{$paddr}
@@ -365,35 +382,37 @@ package Net::BitTorrent;
                 }
             }
             else {
-                my $read  = vec($$rin, $fileno, 1);
-                my $write = vec($$win, $fileno, 1);
+                my $read = (($_max_dl_rate{refaddr $self}
+                             ? max(0,
+                                   (      $_max_dl_rate{refaddr $self}
+                                        - $_k_dl{refaddr $self}
+                                   )
+                                 )
+                             : (2**15)
+                            ) * vec($$rin, $fileno, 1)
+                );
+                my $write = (($_max_ul_rate{refaddr $self}
+                              ? max(0,
+                                    (      $_max_ul_rate{refaddr $self}
+                                         - $_k_ul{refaddr $self}
+                                    )
+                                  )
+                              : (2**15)
+                             ) * vec($$win, $fileno, 1)
+                );
                 my $error = vec($$ein, $fileno, 1)
                     && (   $^E
                         && ($^E != _EINPROGRESS)
                         && ($^E != _EWOULDBLOCK));
-                vec($$rin, $fileno, 1) = 0;
-                vec($$win, $fileno, 1) = 0;
-                vec($$ein, $fileno, 1) = 0;
-                if ($read or $write) {
-                    my ($this_down, $this_up)
+                if ($read || $write || $error) {
+                    my ($this_r, $this_w)
                         = $_connections{refaddr $self}{$fileno}{q[Object]}
-                        ->_rw((($_max_dl_rate{refaddr $self}
-                                ? (($_max_dl_rate{refaddr $self} * 1024)
-                                   - $_k_down{refaddr $self})
-                                : 2**15
-                               ) * $read
-                              ),
-                              (($_max_ul_rate{refaddr $self}
-                                ? (($_max_ul_rate{refaddr $self} * 1024)
-                                   - $_k_up{refaddr $self})
-                                : 2**15
-                               ) * $write
-                              ),
-                              $error
-                        );
-                    $_k_down{refaddr $self}
-                        += defined $this_down ? $this_down : 0;
-                    $_k_up{refaddr $self} += defined $this_up ? $this_up : 0;
+                        ->_rw($read, $write, $error);
+                    $_k_dl{refaddr $self} += defined $this_r ? $this_r : 0;
+                    $_k_ul{refaddr $self} += defined $this_w ? $this_w : 0;
+                    vec($$rin, $fileno, 1) = 0;
+                    vec($$win, $fileno, 1) = 0;
+                    vec($$ein, $fileno, 1) = 0;
                 }
             }
         }
@@ -405,10 +424,9 @@ package Net::BitTorrent;
         my ($self, $infohash) = @_;
         carp q[Bad infohash for Net::BitTorrent->_locate_torrent(INFOHASH)]
             && return
-            if $infohash !~ m[^[\d|a-f]{40}$];
-        return
-            defined $_torrents{refaddr $self}{$infohash}
-            ? $_torrents{refaddr $self}{$infohash}
+            if $infohash !~ m[^[\d|a-f]{40}$]i;
+        return $_torrents{refaddr $self}{lc $infohash}
+            ? $_torrents{refaddr $self}{lc $infohash}
             : undef;
     }
 
@@ -448,17 +466,42 @@ package Net::BitTorrent;
     # Methods | Public | Callback system
     sub on_event {
         my ($self, $type, $method) = @_;
+        carp sprintf q[Unknown callback: %s], $type
+            unless ___check_event($type);
         $_event{refaddr $self}{$type} = $method;
     }
 
     # Methods | Private | Callback system
     sub _event {
         my ($self, $type, $args) = @_;
+        carp sprintf
+            q[Unknown event: %s. This is a bug in Net::BitTorrent; Report it.],
+            $type
+            unless ___check_event($type);
         return $_event{refaddr $self}{$type}
             ? $_event{refaddr $self}{$type}($self, $args)
             : ();
     }
 
+    # Functions | Private | Callback system
+    sub ___check_event {
+        my $type = shift;
+        return scalar grep { $_ eq $type } qw[
+            ip_filter
+            incoming_packet outgoing_packet
+            peer_connect    peer_disconnect
+            peer_read       peer_write
+            tracker_connect tracker_disconnect
+            tracker_read    tracker_write
+            tracker_success tracker_failure
+            piece_hash_pass piece_hash_fail
+            file_open       file_close
+            file_read       file_write
+            file_error
+        ];
+    }
+
+    # Methods | Private | Internal event scheduler
     sub _schedule {
         my ($self, $args) = @_;
         if ((!$args) || (ref $args ne q[HASH])) {
@@ -540,6 +583,7 @@ package Net::BitTorrent;
         my ($self) = @_;
         my @reserved = qw[0 0 0 0 0 0 0 0];
         $reserved[5] |= 0x10;    # Ext Protocol
+        $reserved[7] |= 0x04;    # Fast Ext
         return join q[], map {chr} @reserved;
     }
 
@@ -748,9 +792,7 @@ callbacks.
 Note: This list is subject to change.  Unless mentioned specifically,
 return values from callbacks do not affect behavior.
 
-=over
-
-=item Net::BitTorrent::Peer
+=head2 Net::BitTorrent::Peer
 
 =over
 
@@ -822,7 +864,6 @@ The amount of data, in bytes, sent to the remote peer.
 
 =back
 
-
 =item C<peer_write>
 
 This is triggered whenever we send data to a remote peer via TCP.  The
@@ -840,455 +881,53 @@ The amount of data, in bytes, sent to the remote peer.
 
 =back
 
-=item C<packet_incoming_handshake>
+=item C<outgoing_packet>
 
-Triggered whenever a remote peer has attempted to establish communication
-and has sent a valid handshake according to the current BitTorrent spec.
-The argument hash contains the following keys:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> sending this data.
-
-=item C<Payload>
-
-A list, as returned by
-L<Net::BitTorrent::Protocol|Net::BitTorrent::Protocol/"parse_packet( DATA )">,
-with these values: C<[ReservedBytes, Infohash, PeerID]>.
-
-=back
-
-=item C<packet_outgoing_handshake>
-
-Triggered whenever we attempt to establish a communication with a remote
-peer and have sent a valid handshake according to the current BitTorrent
-spec. The argument hash contains the following keys:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> receiving this data.
-
-=item C<Payload>
-
-A list, as returned by
-L<Net::BitTorrent::Protocol|Net::BitTorrent::Protocol/"parse_packet( DATA )">,
-with these values: C<[ReservedBytes, Infohash, PeerID]>.
-
-=back
-
-=item C<packet_incoming_keepalive>
-
-Triggered when a remote peer has sent us a keepalive packet.  The
-argument hash contains the following key:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> sending this data.
-
-=back
-
-=item C<packet_outgoing_keepalive>
-
-Triggered when we send a keepalive packet to a remote peer.  The
-argument hash contains the following key:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> receiving this data.
-
-=back
-
-=item C<packet_incoming_choke>
-
-Triggered when a remote peer has sent us a choke packet.  The
-argument hash contains the following key:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> sending this data.
-
-=back
-
-=item C<packet_outgoing_choke>
-
-Triggered when we send a remote peer a choke packet.  The argument hash
-contains the following key:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> receiving this data.
-
-=back
-
-=item C<packet_incoming_unchoke>
-
-Triggered when a remote peer has sent us an unchoke packet.  The
-argument hash contains the following key:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> sending this data.
-
-=back
-
-=item C<packet_outgoing_unchoke>
-
-Triggered when we send a remote peer an unchoke packet.  The argument
-hash contains the following key:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> receiving this data.
-
-=back
-
-=item C<packet_incoming_interested>
-
-Triggered when a remote peer has sent us a packet indicating an interest
-in some data we have.  The argument hash contains the following key:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> sending this data.
-
-=back
-
-=item C<packet_outgoing_interested>
-
-Triggered when we send a remote peer a packet indicating an interest
-in some data they have.  The argument hash contains the following key:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> receiving this data.
-
-=back
-
-=item C<packet_incoming_not_interested>
-
-Triggered when a remote peer has sent us a packet indicating they are no
-longer interested.  The argument hash contains the following key:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> sending this data.
-
-=back
-
-=item C<packet_outgoing_not_interested>
-
-Triggered when we send a remote peer a packet indicating they are no
-longer interested.  The argument hash contains the following key:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> receiving this data.
-
-=back
-
-=item C<packet_incoming_have>
-
-Triggered when a remote peer has sent us a packet to let us know they
-have a certain piece available for transmission.  The argument hash
+Triggered when we send a packet to a remote peer.  The argument hash
 contains the following keys:
 
 =over
 
-=item C<Peer>
+=item C<Payload>
 
-The remote L<peer|Net::BitTorrent::Peer> sending this data.
-
-=item C<Index>
-
-The zero based index of the piece the remote peer is advertising.
-
-=back
-
-=item C<packet_outgoing_have>
-
-Triggered when we send a remote peer a packet indicating we have a
-particular piece available for transmission.  The argument hash
-contains the following keys:
-
-=over
+The data sent in the packet (when applicable).
 
 =item C<Peer>
 
 The remote L<peer|Net::BitTorrent::Peer> receiving this data.
 
-=item C<Index>
+=item C<Type>
 
-The zero based index of the piece we advertising.
+The type of packet sent.  These values match the packet types exported
+from L<Net::BitTorrent::Protocol|Net::BitTorrent::Protocol/":types">.
 
 =back
 
-=item C<packet_incoming_bitfield>
+=item C<incoming_packet>
 
-Triggered when a remote peer has sent us a bitfield packet.  The
-argument hash contains the following keys:
+Triggered when we receive a packet to a remote peer.  The argument hash
+contains the following keys:
 
 =over
 
+=item C<Payload>
+
+The data sent in the packet (when applicable).
+
 =item C<Peer>
 
 The remote L<peer|Net::BitTorrent::Peer> sending this data.
 
-=back
+=item C<Type>
 
-=item C<packet_outgoing_bitfield>
-
-Triggered when we send a remote peer a bitfield packet representing the
-pieces we have available for transmission.  The argument hash contains
-the following keys:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> receiving this data.
+The type of packet sent.  These values match the packet types exported
+from L<Net::BitTorrent::Protocol|Net::BitTorrent::Protocol/":types">.
 
 =back
 
-=item C<packet_incoming_request>
-
-Triggered when a remote peer has sent us a packet requesting a sub-piece
-(or block) of data.  The argument hash contains the following keys:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> sending this request.
-
-=item C<Index>
-
-The zero-based index of the piece in which this particular block is
-located.
-
-=item C<Offset>
-
-The offset, in bytes, from the beginning of the aforementioned piece
-where the requested data is located.
-
-=item C<Length>
-
-The amount of data requested in bytes.
-
 =back
 
-=item C<packet_outgoing_request>
-
-Triggered when we request a sub-piece (or block) of data from a remote
-peer.  The argument hash contains the following keys:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> receiving this request.
-
-=item C<Index>
-
-The zero-based index of the piece in which this particular block is
-located.
-
-=item C<Offset>
-
-The offset, in bytes, from the beginning of the aforementioned piece
-where the requested data is located.
-
-=item C<Length>
-
-The amount of data requested in bytes.
-
-=back
-
-=item C<packet_incoming_block>
-
-Triggered when a remote peer has sent us a packet containing a sub-piece
-(or block) of data we previously requested from them.  The argument hash
-contains the following keys:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> sending this request.
-
-=item C<Index>
-
-The zero-based index of the piece in which this particular block is
-located.
-
-=item C<Offset>
-
-The offset, in bytes, from the beginning of the aforementioned piece
-where the requested data is located.
-
-=item C<Length>
-
-The amount of data requested in bytes.
-
-=back
-
-=item C<packet_outgoing_block>
-
-Triggered when we send a remote peer a packet containing a sub-piece
-(or block) of data they previously requested.  The argument hash
-contains the following keys:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> receiving this request.
-
-=item C<Index>
-
-The zero-based index of the piece in which this particular block is
-located.
-
-=item C<Offset>
-
-The offset, in bytes, from the beginning of the aforementioned piece
-where the requested data is located.
-
-=item C<Length>
-
-The amount of data requested in bytes.
-
-=back
-
-=item C<packet_incoming_cancel>
-
-Triggered when a remote peer has sent us a packet canceling a request
-they made.  The argument hash contains the following keys:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> sending this packet.
-
-=item C<Index>
-
-The zero-based index of the piece in which this particular block is
-located.
-
-=item C<Offset>
-
-The offset, in bytes, from the beginning of the aforementioned piece
-where the requested data is located.
-
-=item C<Length>
-
-The amount of data originally requested in bytes.
-
-=back
-
-=item C<packet_outgoing_cancel>
-
-Triggered when we cancel a request made for a certain sub-piece (or
-block) of data.  The argument hash contains the following keys:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> on the receiving end of this
-packet.
-
-=item C<Index>
-
-The zero-based index of the piece in which this particular block is
-located.
-
-=item C<Offset>
-
-The offset, in bytes, from the beginning of the aforementioned piece
-where the requested data is located.
-
-=item C<Length>
-
-The amount of data originally requested in bytes.
-
-=back
-
-=item C<packet_incoming_extended>
-
-Triggered when a remote peer has sends us a packet containing extended
-protocol data.  The argument hash contains the following keys:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> sending this packet.
-
-=item C<ID>
-
-The numeric ID for this packet.
-
-=item C<Payload>
-
-The data sent as the body of this message.
-
-=back
-
-Please see the official documentation for more:
-http://bittorrent.org/beps/bep_0010.html
-
-=item C<packet_outgoing_extended>
-
-Triggered when we send an extended protocol packet to a remote peer.
-The argument hash contains the following keys:
-
-=over
-
-=item C<Peer>
-
-The remote L<peer|Net::BitTorrent::Peer> receiving this packet.
-
-=item C<ID>
-
-The numeric ID for this packet.
-
-=item C<Payload>
-
-The data sent as the body of this message.
-
-=back
-
-Please see the official documentation for more:
-http://bittorrent.org/beps/bep_0010.html
-
-=back
-
-=item Net::BitTorrent::Torrent::File
+=head2 Net::BitTorrent::Torrent::File
 
 =over
 
@@ -1376,7 +1015,7 @@ The actual amount of data written to the file.
 
 =back
 
-=item Net::BitTorrent::Torrent::Tracker::HTTP/Net::BitTorrent::Torrent::Tracker::UDP
+=head2 Net::BitTorrent::Torrent::Tracker::HTTP/Net::BitTorrent::Torrent::Tracker::UDP
 
 Note: The tracker objects passed to these callbacks will either be a
 L<Net::BitTorrent::Torrent::Tracker::HTTP|Net::BitTorrent::Torrent::Tracker::HTTP>
@@ -1516,7 +1155,7 @@ The amount of data received from the remote tracker.
 
 =back
 
-=item Net::BitTorrent::Torrent
+=head2 Net::BitTorrent::Torrent
 
 =over
 
@@ -1558,8 +1197,6 @@ metadata.
 
 =back
 
-=back
-
 =head1 Bugs
 
 Numerous, I'm sure.
@@ -1686,6 +1323,6 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 Neither this module nor the L<Author|/Author> is affiliated with
 BitTorrent, Inc.
 
-=for svn $Id: BitTorrent.pm 33 2008-11-10 23:27:24Z sanko@cpan.org $
+=for svn $Id: BitTorrent.pm 34 2008-11-20 03:38:52Z sanko@cpan.org $
 
 =cut
