@@ -9,8 +9,8 @@ package Net::BitTorrent::DHT::Node;
     use lib q[../../../../lib/];
     use Net::BitTorrent::Util qw[:bencode :compact];
     use version qw[qv];
-    our $SVN = q[$Id: Node.pm 33 2008-11-10 23:27:24Z sanko@cpan.org $];
-    our $UNSTABLE_RELEASE = 0; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 33 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
+    our $SVN = q[$Id: Node.pm 34 2008-11-20 03:38:52Z sanko@cpan.org $];
+    our $UNSTABLE_RELEASE = 0; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 34 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
     my (@CONTENTS)
         = \my (%dht,        %packed_host,   %node_id,
                %added,      %infohashes,    %_token,
@@ -303,9 +303,10 @@ package Net::BitTorrent::DHT::Node;
                 }
             }
             else {
-                warn sprintf
-                    q[...torrent '%s' not found!?!?!?!?!?!?!?!?],
-                    unpack q[H*], $packet->{q[r]}{q[token]};
+
+                #warn sprintf
+                #    q[...torrent '%s' not found!?!?!?!?!?!?!?!?],
+                #    unpack q[H*], $packet->{q[r]}{q[token]};
             }
             return 1;
         }
@@ -591,6 +592,6 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 Neither this module nor the L<Author|/Author> is affiliated with
 BitTorrent, Inc.
 
-=for svn $Id: Node.pm 33 2008-11-10 23:27:24Z sanko@cpan.org $
+=for svn $Id: Node.pm 34 2008-11-20 03:38:52Z sanko@cpan.org $
 
 =cut
