@@ -3,19 +3,17 @@ package Net::BitTorrent::Version;
 {
     use strict;
     use warnings;
-
-    #
     use version qw[qv];
-    our $SVN = q[$Id: Version.pm 33 2008-11-10 23:27:24Z sanko@cpan.org $];
-    our $VERSION_BASE = 27; our $UNSTABLE_RELEASE = 11; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new(($VERSION_BASE))->numify / 1000), $UNSTABLE_RELEASE);
-    our $PRODUCT_TOKEN = qq[Net::BitTorrent/$VERSION ($^O)];    # ext protocol
+    our $SVN = q[$Id: Version.pm 34 2008-11-20 03:38:52Z sanko@cpan.org $];
+    our $VERSION_BASE = 27; our $UNSTABLE_RELEASE = 15; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new(($VERSION_BASE))->numify / 1000), $UNSTABLE_RELEASE);
+    our $PRODUCT_TOKEN = qq[Net::BitTorrent $VERSION];
 
     sub gen_peerid {
         return pack(
             q[a20],
             (sprintf(
                  q[NB%03d%1s-%8s%5s],
-                 $VERSION_BASE,      # formerly: (q[$Rev: 33 $] =~ m[(\d+)]g),
+                 $VERSION_BASE,
                  ($UNSTABLE_RELEASE ? q[U] : q[S]),
                  (join q[],
                   map {
@@ -29,7 +27,7 @@ package Net::BitTorrent::Version;
         );
     }
 
-    sub gen_node_id {    # DHT
+    sub gen_node_id {
         return
             pack(q[a20],
                  (join(q[],
@@ -188,6 +186,6 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 Neither this module nor the L<Author|/Author> is affiliated with
 BitTorrent, Inc.
 
-=for svn $Id: Version.pm 33 2008-11-10 23:27:24Z sanko@cpan.org $
+=for svn $Id: Version.pm 34 2008-11-20 03:38:52Z sanko@cpan.org $
 
 =cut
